# {{cookiecutter.project_name}}

更美私信RPC服务

## 本地部署

1. 环境准备

```sh
# ready file: /etc/gm-config
mkdir -p /data/log/gm_im/app
virtualenv .venv -p python3.8
source .venv/bin/activate
# 创建虚拟环境，安装依赖
pip install -r requirements.txt
```


2. 启动项目


```sh
# dev (reload)
RPCD_SETTINGS_MODULE='{{cookiecutter.settings_dir}}.settings' uvicorn fastapi_rpcd.asgi:application --port 8000 --reload
# deploy
RPCD_SETTINGS_MODULE='{{cookiecutter.settings_dir}}.settings' gunicorn fastapi_rpcd.asgi:application -w 3 -b 0.0.0.0:8000 -k 'uvicorn.workers.UvicornWorker'
```

## 本地开发

1. 安装 pre-commit/pylint/tox 依赖
2. 在本地执行 `pre-commit install`
3. 在进行代码提交(git commit)的时候，会进行代码格式校验
4. 本地执行单元测试: `tox`

## 数据/设计/版本特性文档

```sh
mkdocs serve
```


## 生成/查看API文档

```sh
npm install apidoc -g
apidoc -i {{cookiecutter.first_app_name}}/views -o apidoc/
```

## PaaS 部署

1. 环境准备

- paas-conf 项目配置 `.env` 文件准备
- configs 项目 配置文件准备

2. 线上 Sentry 项目开通

3. 容器启动方式见 `Dockerfile`



## 项目背景

{{cookiecutter.project_local_name}}

## 类库管理

* 时间处理 arrow https://github.com/crsmithdev/arrow
* schema管理 pydantic https://pydantic-docs.helpmanual.io/
* url管理 Starlette https://www.starlette.io/
* http请求 aiohttp https://github.com/aio-libs/aiohttp
* 文档 apidoc https://apidocjs.com/
* 部署 uvicorn http://www.uvicorn.org/
* 日志 loguru https://github.com/Delgan/loguru
* 单元测试 pytest https://github.com/pytest-dev/pytest
* kafka客户端 aiokafka https://github.com/aio-libs/aiokafka
* redis客户端 aioredis https://github.com/aio-libs/aioredis
* mysql客户端 aiomysql https://github.com/aio-libs/aiomysql
* elasticsearch客户端 aioelasticsearch https://github.com/aio-libs/aioelasticsearch


## 分支管理

- 创建需求卡片或者缺陷卡片
- 根据卡片，从迭代分支(master)拉出新分支，区分功能分支或补丁分支。功能分支命名规范为task/\{\{issue_id\}\}，补丁分支命名规范为bug/\{\{card_id\}\}
- 新分支开发完成后，或者需要讨论的时候，就向迭代分支发起一个merge request（简称PR）。
- merge Request既是一个通知，让别人注意到你的请求，又是一种对话机制，大家一起评审和讨论你的代码。对话过程中，你还可以不断提交代码。需要填写reviewer同学
- 你的merge Request被接受，合并进迭代分支(master)，重新部署后，原来你拉出来的那个分支就被删除。（先部署再合并也可。）


## 单元测试

- CI 增加合并前的检查:
    1. 代码规范检查检查: pre-commit + pylint
    2. 单元测试用例执行:
       1. 有失败的 Case 时候，不能合并到共用分支上
       2. 覆盖率检查, 低于 80% 视作单元测试运行不通过

- 测试用例 QuickStart:
  - 执行: tox -e py38
  - 示例: tests/view/test_demo.py
  
  
## postgre全文检索

参靠文档：[https://www.jianshu.com/p/0baedb2c550d](https://www.jianshu.com/p/0baedb2c550d)

生成解释器，全局配置，只需执行一次：
CREATE EXTENSION zhparser;
CREATE TEXT SEARCH CONFIGURATION testzhcfg(PARSER=zhparser);
ALTER TEXT SEARCH CONFIGURATION testzhcfg ADD MAPPING FOR n,v,a,i,e,l WITH simple;
alter role all set zhparser.multi_short=on;
alter role all set zhparser.punctuation_ignore=on;

创建gin索引：
create index idx_stream_user_view_lastname_gin on stream_user_view using gin(to_tsvector('testzhcfg',coalesce(lastname,'')));

使用gin索引：
select * from stream_user_view where to_tsvector('testzhcfg',coalesce(lastname,'')) @@ to_tsquery('testzhcfg','伟力');
