# -*- coding: utf-8 -*-
import logging

from pydantic.utils import import_string

LOGGING_CONFIG = 'logging.config.dictConfig'


class RequireDebugFalse(logging.Filter):
    def filter(self, record):
        from rpcd.conf import settings
        return not settings.DEBUG


class RequireDebugTrue(logging.Filter):
    def filter(self, record):
        from rpcd.conf import settings
        return settings.DEBUG


DEFAULT_LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'rpcd.conf.log.RequireDebugFalse',
        },
        'require_debug_true': {
            '()': 'rpcd.conf.log.RequireDebugTrue',
        },
    },
    'formatters': {
        'rpcd': {
            'format': '%(asctime)s %(levelname)s %(module)s.%(funcName)s Line:%(lineno)d  %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'rpcd': {
            'handlers': ['console'],
            'level': 'INFO',
        },
    }
}


def configure_logging(logging_config, logging_settings):
    if logging_config:
        # First find the logging configuration function ...
        logging_config_func = import_string(logging_config)

        logging.config.dictConfig(DEFAULT_LOGGING)

        # ... then invoke it with the logging settings
        if logging_settings:
            logging_config_func(logging_settings)
