# -*- coding: utf-8 -*-
from functools import partial

import aioredis
import six
from aioredis.abc import AbcPool

from rpcd.conf import settings
from rpcd.functional import SimpleLazyObject

redis:AbcPool = None

async def initialize_redis_pool():
    global redis
    redis = await aioredis.create_redis_pool(settings.REDIS_DSN)

async def release_redis_pool():
    global redis
    await redis.wait_closed()


class PrefixRedis(object):

    CLIENT_KEY = '_client'

    # add methods those are need to be hacked here
    _hacked_methods = [
        # key
        'delete', 'exists', 'expire', 'expireat',
        # string
        'append', 'decr', 'decrby', 'get', 'getset', 'incr', 'incrby', 'incrbyfloat',
        'mget', 'mset', 'msetnx', 'set', 'setex', 'setnx', 'setrange', 'strlen',
        # hash
        'hdel', 'hexists', 'hget', 'hgetall', 'hincrby', 'hincrbyfloat', 'hkeys', 'hlen',
        'hmget', 'hmset', 'hset', 'hsetnx', 'hvals',
        # list
        'blpop', 'brpop', 'brpoplpush', 'lindex', 'linsert', 'llen', 'lpop', 'lpush',
         'lpushx', 'lrange', 'lrem', 'lset', 'ltrim', 'rpop', 'rpoplpush', 'rpush',
         'rpushx',
        # set
        'sadd', 'scard', 'sdiff', 'sdiffstore', 'sinter', 'sinterstore', 'sismember',
         'smembers', 'smove', 'spop', 'srandmember', 'srem', 'sunion', 'sunionstore',
         'sscan',
        # sortedset
        'zadd', 'zcard', 'zcount', 'zincrby', 'zrange', 'zrangebyscore', 'zrank',
         'zrem', 'zremrangebyrank', 'zremrangebyscore', 'zrevrange', 'zrevrangebyscore',
         'zrevrank', 'zscore', 'zunionstore', 'zinterstore',
    ]

    def __getattribute__(self, name):
        try:
            original_client = object.__getattribute__(self, PrefixRedis.CLIENT_KEY)
            attr = getattr(original_client, name)
        except AttributeError:
            return object.__getattribute__(self, name)

        if name in PrefixRedis._hacked_methods:
            def newfunc(key, *args, **kwargs):
                prefix = object.__getattribute__(self, 'prefix')
                if isinstance(key, six.string_types):
                    k = prefix + ':' + key
                elif isinstance(key, list):
                    k = [prefix + ':' + _k for _k in key]
                else:
                    raise TypeError("must be str or list")
                result = attr(k, *args, **kwargs)
                return result

            return newfunc

        return attr

    def __init__(self, prefix, client):
        self.prefix = prefix
        self._client = client

    def get_full_key_path(self, key):
        return self.prefix + ':' + key

    @property
    def original_client(self):
        return self._client

    @classmethod
    def get_instance(cls, prefix='', cache_alias='default'):
        global redis
        return cls(prefix, redis)

_redis_proxy = PrefixRedis.get_instance
# Instance Example:
# unread_cache = SimpleLazyObject(partial(_redis_proxy, 'unread'))